@@ -8,18 +8,22 @@
  *
  * Contains functions used to do authentication.
  *
- * $Id: auth.php,v 1.14 2002/01/18 11:53:18 philippe_mingo Exp $
+ * $Id: auth.php,v 1.18 2002/02/22 10:15:18 brong Exp $
  */
 
 require_once( '../functions/page_header.php' );
 
 function is_logged_in () {
-    global $squirrelmail_language;
+    global $squirrelmail_language, $frame_top, $base_uri;
 
     if ( session_is_registered('user_is_logged_in') ) {
         return;
     }
 
+    if (!isset($frame_top) || $frame_top == '' ) {
+        $frame_top = '_top';
+    }
+
     set_up_language($squirrelmail_language, true);
 
     displayHtmlHeader( 'SquirrelMail', '', FALSE );
@@ -27,9 +31,9 @@ function is_logged_in () {
     echo "<body bgcolor=\"ffffff\">\n" .
          '<br><br><center><b>' .
          _("You must be logged in to access this page.").'</b><br><br>' .
-         "<a href=\"../src/login.php\" target=\"_top\">"._("Go to the login page")."</a>\n" .
+         "<a href=\"$base_uri" . "src/login.php\" target=\"$frame_top\">"._("Go to the login page")."</a>\n" .
          "</center></body></html>\n";
     exit;
 }
 
-?>
\ No newline at end of file
+?>
