@@ -9,7 +9,7 @@
  * This contains all the functions needed to send messages through
  * an smtp server or sendmail.
  *
- * $Id: smtp.php,v 1.141 2002/01/07 15:59:54 ullgren Exp $
+ * $Id: smtp.php,v 1.146 2002/02/13 21:02:00 jmunro Exp $
  */
 
 require_once('../functions/addressbook.php');
@@ -108,13 +108,13 @@ function expandRcptAddrs ($array) {
 }
 
 
-/* Attach the files that are due to be attached 
+/* Attach the files that are due to be attached
  */
 function attachFiles ($fp) {
     global $attachments, $attachment_dir, $username;
-    
+
     $length = 0;
-    
+
     $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
     if (isMultipart()) {
         foreach ($attachments as $info) {
@@ -124,21 +124,26 @@ function attachFiles ($fp) {
             else {
                 $filetype = 'application/octet-stream';
             }
-            
-            $header = '--'.mimeBoundary()."\r\n";
-            $header .= "Content-Type: $filetype; name=\"" .
-                $info['remotefilename'] . "\"\r\n";
-            $header .= "Content-Disposition: attachment; filename=\"" .
-                $info['remotefilename'] . "\"\r\n";
-                
-            /* Use 'rb' for NT systems -- read binary 
-             * Unix doesn't care -- everything's binary!  :-) 
+
+            $header = '--' . mimeBoundary() . "\r\n";
+            if ( isset($info['remotefilename']) && $info['remotefilename'] != '') {
+                $header .= "Content-Type: $filetype; name=\"" .
+                    $info['remotefilename'] . "\"\r\n";
+                $header .= "Content-Disposition: attachment; filename=\"" .
+                    $info['remotefilename'] . "\"\r\n";
+            } else {
+                $header .= "Content-Type: $filetype;\r\n";
+            }
+
+
+            /* Use 'rb' for NT systems -- read binary
+             * Unix doesn't care -- everything's binary!  :-)
              */
-            
+
             $filename = $hashed_attachment_dir . '/' . $info['localfilename'];
             $file = fopen ($filename, 'rb');
             if (substr($filetype, 0, 5) == 'text/' ||
-                $filetype == 'message/rfc822') {
+                substr($filetype, 0, 8) == 'message/' ) {
                 $header .= "\r\n";
                 fputs ($fp, $header);
                 $length += strlen($header);
@@ -190,12 +195,13 @@ function deleteAttachments() {
  */
 function mimeBoundary () {
     static $mimeBoundaryString;
-    
-    if ($mimeBoundaryString == "") {
-        $mimeBoundaryString = "----=_" . 
-            GenerateRandomString(60, '\'()+,-./:=?_', 7);
+
+    if ( !isset( $mimeBoundaryString ) ||
+         $mimeBoundaryString == '') {
+        $mimeBoundaryString = '----=_' . date( 'YmdHis' ) . '_' .
+                              mt_rand( 10000, 99999 );
     }
-    
+
     return $mimeBoundaryString;
 }
 
@@ -281,7 +287,7 @@ function write822Header ($fp, $t, $c, $b, $subject, $more_headers) {
         else {
             $received_from = $REMOTE_ADDR;
         }
-        
+
         if (isset($HTTP_VIA) || isset ($HTTP_X_FORWARDED_FOR)) {
             if ($HTTP_X_FORWARDED_FOR == '') {
                 $HTTP_X_FORWARDED_FOR = 'unknown';
@@ -301,8 +307,26 @@ function write822Header ($fp, $t, $c, $b, $subject, $more_headers) {
         $header .= "From: $from\r\n";
         $header .= "To: $to_list\r\n";    // Who it's TO
         
-        /* Insert headers from the $more_headers array */
-        if(is_array($more_headers)) {
+        if (isset($more_headers["Content-Type"])) {
+	  $contentType = $more_headers["Content-Type"];
+	  unset($more_headers["Content-Type"]);
+	}
+        else {
+	  if (isMultipart()) {
+	    $contentType = "multipart/mixed;";
+	  }
+	  else {
+	    if ($default_charset != '') {
+                $contentType = 'text/plain; charset='.$default_charset;
+            }
+            else {
+                $contentType = 'text/plain;';
+            } 
+	  }
+        }
+            
+	/* Insert headers from the $more_headers array */
+	if(is_array($more_headers)) {
             reset($more_headers);
             while(list($h_name, $h_val) = each($more_headers)) {
                 $header .= sprintf("%s: %s\r\n", $h_name, $h_val);
@@ -325,21 +349,16 @@ function write822Header ($fp, $t, $c, $b, $subject, $more_headers) {
         }
         
         $header .= "X-Mailer: SquirrelMail (version $version)\r\n"; /* Identify SquirrelMail */
-        
+
         /* Do the MIME-stuff */
         $header .= "MIME-Version: 1.0\r\n";
         
         if (isMultipart()) {
-            $header .= 'Content-Type: multipart/mixed; boundary="';
+            $header .= 'Content-Type: '.$contentType.' boundary="';
             $header .= mimeBoundary();
             $header .= "\"\r\n";
         } else {
-            if ($default_charset != '') {
-                $header .= "Content-Type: text/plain; charset=$default_charset\r\n";
-            }
-            else {
-                $header .= "Content-Type: text/plain;\r\n";
-            }
+	    $header .= 'Content-Type: '.$contentType."\r\n";
             $header .= "Content-Transfer-Encoding: 8bit\r\n";
         }
         $header .= "\r\n"; // One blank line to separate header and body
@@ -457,7 +476,9 @@ function sendSMTP($t, $c, $b, $subject, $body, $more_headers) {
         exit;
     }
     $tmp = fgets($smtpConnection, 1024);
-    if (errorCheck($tmp, $smtpConnection)!=5) return(0);
+    if (errorCheck($tmp, $smtpConnection)!=5) {
+        return(0);
+    }
     
     $to_list = getLineOfAddrs($to);
     $cc_list = getLineOfAddrs($cc);
@@ -477,7 +498,7 @@ function sendSMTP($t, $c, $b, $subject, $body, $more_headers) {
         if (errorCheck($tmp, $smtpConnection)!=5) {
             return(0);
         }
-        
+
         fputs($smtpConnection, base64_encode ($username) . "\r\n");
         $tmp = fgets($smtpConnection, 1024);
         if (errorCheck($tmp, $smtpConnection)!=5) {
@@ -516,18 +537,18 @@ function sendSMTP($t, $c, $b, $subject, $body, $more_headers) {
     for ($i = 0; $i < count($bcc); $i++) {
         fputs($smtpConnection, "RCPT TO: $bcc[$i]\r\n");
         $tmp = fgets($smtpConnection, 1024);
-        if (errorCheck($tmp, $smtpConnection)!=5) { 
+        if (errorCheck($tmp, $smtpConnection)!=5) {
             return(0);
         }
     }
-    
+
     /* Lets start sending the actual message */
     fputs($smtpConnection, "DATA\r\n");
     $tmp = fgets($smtpConnection, 1024);
     if (errorCheck($tmp, $smtpConnection)!=5) {
         return(0);
     }
-    
+
     /* Send the message */
     $headerlength = write822Header ($smtpConnection, $t, $c, $b, $subject, $more_headers);
     $bodylength = writeBody($smtpConnection, $body);
@@ -536,11 +557,6 @@ function sendSMTP($t, $c, $b, $subject, $body, $more_headers) {
     $tmp = fgets($smtpConnection, 1024);
     $num = errorCheck($tmp, $smtpConnection, true);
     if ($num != 250) {
-        $tmp = nl2br(htmlspecialchars($tmp));
-        displayPageHeader($color, 'None');
-        include_once('../functions/display_messages.php');
-        $msg  = "Message not sent!<br>\nReason given: $tmp";
-        plain_error_message($msg, $color);
         return(0);
     }
     
@@ -553,7 +569,7 @@ function sendSMTP($t, $c, $b, $subject, $body, $more_headers) {
 
 
 function errorCheck($line, $smtpConnection, $verbose = false) {
-    global $color;
+    global $color, $compose_new_win;
     
     /* Read new lines on a multiline response */
     $lines = $line;
@@ -645,7 +661,12 @@ function errorCheck($line, $smtpConnection, $verbose = false) {
     
     if ($status == 0) {
         include_once('../functions/page_header.php');
-        displayPageHeader($color, 'None');
+        if ($compose_new_win == '1') {
+            compose_Header($color, 'None');
+        }
+		else {
+            displayPageHeader($color, 'None');
+        }
         include_once('../functions/display_messages.php');
         $lines = nl2br(htmlspecialchars($lines));
         $msg  = $message . "<br>\nServer replied: $lines";
@@ -655,22 +676,22 @@ function errorCheck($line, $smtpConnection, $verbose = false) {
     return $err_num;
 }
 
-function sendMessage($t, $c, $b, $subject, $body, $reply_id, $prio = 3) {
-    global $useSendmail, $msg_id, $is_reply, $mailbox, $onetimepad;
-    global $data_dir, $username, $domain, $key, $version, $sent_folder, $imapServerAddress, $imapPort;
-    global $default_use_priority;
-    global $more_headers;
+function sendMessage($t, $c, $b, $subject, $body, $reply_id, $MDN, $prio = 3) {
+    global $useSendmail, $msg_id, $is_reply, $mailbox, $onetimepad,
+           $data_dir, $username, $domain, $key, $version, $sent_folder, $imapServerAddress, 
+           $imapPort, $default_use_priority, $more_headers, $request_mdn, $request_dr;
+
     $more_headers = Array();
     
-    do_hook("smtp_send");
-    
+    do_hook('smtp_send');
+
     $imap_stream = sqimap_login($username, $key, $imapServerAddress, $imapPort, 1);
-    
+
     if (isset($reply_id) && $reply_id) {
         sqimap_mailbox_select ($imap_stream, $mailbox);
         sqimap_messages_flag ($imap_stream, $reply_id, $reply_id, 'Answered');
-        
-        /* Insert In-Reply-To and References headers if the 
+
+        /* Insert In-Reply-To and References headers if the
          * message-id of the message we reply to is set (longer than "<>")
          * The References header should really be the old Referenced header
          * with the message ID appended, but it can be only the message ID too.
@@ -684,26 +705,41 @@ function sendMessage($t, $c, $b, $subject, $body, $reply_id, $prio = 3) {
     if ($default_use_priority) {
         $more_headers = array_merge($more_headers, createPriorityHeaders($prio));
     }
-    
+
+    $requestRecipt = 0;
+    if (isset($request_dr)) {
+        $requestRecipt += 1;
+    }
+    if (isset($request_mdn)) {
+        $requestRecipt += 2;
+    }
+    if ( $requestRecipt > 0) {
+        $more_headers = array_merge($more_headers, createReceiptHeaders($requestRecipt));
+    }
+
     /* In order to remove the problem of users not able to create
      * messages with "." on a blank line, RFC821 has made provision
      * in section 4.5.2 (Transparency).
      */
     $body = ereg_replace("\n\\.", "\n..", $body);
     $body = ereg_replace("^\\.", "..", $body);
-    
+
     /* this is to catch all plain \n instances and
      * replace them with \r\n.  All newlines were converted
      * into just \n inside the compose.php file.
      */
     $body = ereg_replace("\n", "\r\n", $body);
     
+    if ($MDN) {
+        $more_headers["Content-Type"] = "multipart/report; ".
+            "report-type=disposition-notification;";
+    }
+
     if ($useSendmail) {
-        $length = sendSendmail($t, $c, $b, $subject, $body, $more_headers);
+        $length = sendSendmail($t, $c, $b, $subject, $body,  $more_headers);
     } else {
         $length = sendSMTP($t, $c, $b, $subject, $body, $more_headers);
     }
-    
     if (sqimap_mailbox_exists ($imap_stream, $sent_folder)) {
         sqimap_append ($imap_stream, $sent_folder, $length);
         write822Header ($imap_stream, $t, $c, $b, $subject, $more_headers);
@@ -714,31 +750,64 @@ function sendMessage($t, $c, $b, $subject, $body, $reply_id, $prio = 3) {
     /* Delete the files uploaded for attaching (if any).
      * only if $length != 0 (if there was no error)
      */
-    if ($length)
+    if ($length) {
         ClearAttachments();
-    
+    }
+
     return $length;
 }
 
 function createPriorityHeaders($prio) {
     $prio_headers = Array();
-    $prio_headers["X-Priority"] = $prio;
-    
+    $prio_headers['X-Priority'] = $prio;
+
     switch($prio) {
-    case 1: $prio_headers["Importance"] = "High";
-        $prio_headers["X-MSMail-Priority"] = "High";
+    case 1: $prio_headers['Importance'] = 'High';
+        $prio_headers['X-MSMail-Priority'] = 'High';
         break;
-        
-    case 3: $prio_headers["Importance"] = "Normal";
-        $prio_headers["X-MSMail-Priority"] = "Normal";
+
+    case 3: $prio_headers['Importance'] = 'Normal';
+        $prio_headers['X-MSMail-Priority'] = 'Normal';
         break;
-        
+
     case 5:
-        $prio_headers["Importance"] = "Low";
-        $prio_headers["X-MSMail-Priority"] = "Low";
+        $prio_headers['Importance'] = 'Low';
+        $prio_headers['X-MSMail-Priority'] = 'Low';
         break;
     }
     return  $prio_headers;
 }
 
+function createReceiptHeaders($receipt) {
+
+    GLOBAL $data_dir, $username;
+
+    $receipt_headers = Array();
+    $from_addr = getPref($data_dir, $username, 'email_address');
+    $from = getPref($data_dir, $username, 'full_name');
+
+    if ($from == '') {
+        $from = "<$from_addr>";
+    }
+    else {
+        $from = '"' . encodeHeader($from) . "\" <$from_addr>";
+    }
+
+    /* On Delivery */
+    if ( $receipt == 1
+        || $receipt == 3 ) {
+        $receipt_headers["Return-Receipt-To"] = $from;
+    }
+    /* On Read */
+    if ($receipt == 2
+        || $receipt == 3 ) {
+        /* Pegasus Mail */
+        $receipt_headers["X-Confirm-Reading-To"] = $from;
+        /* RFC 2298 */
+        $receipt_headers["Disposition-Notification-To"] = $from;
+    }
+    return $receipt_headers;
+}
+
+
 ?>
